@@ -43,16 +43,10 @@
 #endif
 #include <string.h>
 #include <math.h>
+#include <gst/video/video.h>
 
 #include "gstdvdec.h"
 
-
-static const GstElementDetails dvdec_details =
-GST_ELEMENT_DETAILS ("DV video decoder",
-    "Codec/Decoder/Video",
-    "Uses libdv to decode DV video (smpte314) (libdv.sourceforge.net)",
-    "Erik Walthinsen <omega@cse.ogi.edu>," "Wim Taymans <wim@fluendo.com>");
-
 /* sizes of one input buffer */
 #define NTSC_HEIGHT 480
 #define NTSC_BUFFER 120000
@@ -131,7 +125,7 @@ static GstStaticPadTemplate src_temp = GST_STATIC_PAD_TEMPLATE ("src",
     );
 
 #define GST_TYPE_DVDEC_QUALITY (gst_dvdec_quality_get_type())
-GType
+static GType
 gst_dvdec_quality_get_type (void)
 {
   static GType qtype = 0;
@@ -177,7 +171,10 @@ gst_dvdec_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_temp));
 
-  gst_element_class_set_details (element_class, &dvdec_details);
+  gst_element_class_set_details_simple (element_class, "DV video decoder",
+      "Codec/Decoder/Video",
+      "Uses libdv to decode DV video (smpte314) (libdv.sourceforge.net)",
+      "Erik Walthinsen <omega@cse.ogi.edu>," "Wim Taymans <wim@fluendo.com>");
 
   GST_DEBUG_CATEGORY_INIT (dvdec_debug, "dvdec", 0, "DV decoding element");
 }
@@ -328,7 +325,7 @@ gst_dvdec_src_negotiate (GstDVDec * dvdec)
       "framerate", GST_TYPE_FRACTION, dvdec->framerate_numerator,
       dvdec->framerate_denominator,
       "pixel-aspect-ratio", GST_TYPE_FRACTION, dvdec->par_x,
-      dvdec->par_y, NULL);
+      dvdec->par_y, "interlaced", G_TYPE_BOOLEAN, dvdec->interlaced, NULL);
 
   gst_pad_set_caps (dvdec->srcpad, othercaps);
   gst_caps_unref (othercaps);
@@ -434,6 +431,7 @@ gst_dvdec_chain (GstPad * pad, GstBuffer * buf)
 
   dvdec->height = (dvdec->PAL ? PAL_HEIGHT : NTSC_HEIGHT);
 
+  dvdec->interlaced = !dv_is_progressive (dvdec->decoder);
 
   /* negotiate if not done yet */
   if (!dvdec->src_negotiated) {
@@ -466,6 +464,8 @@ gst_dvdec_chain (GstPad * pad, GstBuffer * buf)
   dv_decode_full_frame (dvdec->decoder, inframe,
       e_dv_color_yuv, outframe_ptrs, outframe_pitches);
 
+  GST_BUFFER_FLAG_UNSET (outbuf, GST_VIDEO_BUFFER_TFF);
+
   GST_BUFFER_OFFSET (outbuf) = GST_BUFFER_OFFSET (buf);
   GST_BUFFER_OFFSET_END (outbuf) = GST_BUFFER_OFFSET_END (buf);
   GST_BUFFER_TIMESTAMP (outbuf) = cstart;
