@@ -50,12 +50,6 @@
 GST_DEBUG_CATEGORY_STATIC (speexdec_debug);
 #define GST_CAT_DEFAULT speexdec_debug
 
-static const GstElementDetails speex_dec_details =
-GST_ELEMENT_DETAILS ("Speex audio decoder",
-    "Codec/Decoder/Audio",
-    "decode speex streams to audio",
-    "Wim Taymans <wim@fluendo.com>");
-
 #define DEFAULT_ENH   TRUE
 
 enum
@@ -115,7 +109,9 @@ gst_speex_dec_base_init (gpointer g_class)
       gst_static_pad_template_get (&speex_dec_src_factory));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&speex_dec_sink_factory));
-  gst_element_class_set_details (element_class, &speex_dec_details);
+  gst_element_class_set_details_simple (element_class, "Speex audio decoder",
+      "Codec/Decoder/Audio",
+      "decode speex streams to audio", "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -144,7 +140,6 @@ static void
 gst_speex_dec_reset (GstSpeexDec * dec)
 {
   gst_segment_init (&dec->segment, GST_FORMAT_UNDEFINED);
-  dec->granulepos = -1;
   dec->packetno = 0;
   dec->frame_size = 0;
   dec->frame_duration = 0;
@@ -152,6 +147,12 @@ gst_speex_dec_reset (GstSpeexDec * dec)
   free (dec->header);
   dec->header = NULL;
   speex_bits_destroy (&dec->bits);
+
+  if (dec->stereo) {
+    speex_stereo_state_destroy (dec->stereo);
+    dec->stereo = NULL;
+  }
+
   if (dec->state) {
     speex_decoder_destroy (dec->state);
     dec->state = NULL;
@@ -475,8 +476,6 @@ speex_dec_sink_event (GstPad * pad, GstEvent * event)
       gst_segment_set_newsegment_full (&dec->segment, update,
           rate, arate, GST_FORMAT_TIME, start, stop, time);
 
-      dec->granulepos = -1;
-
       GST_DEBUG_OBJECT (dec, "segment now: cur = %" GST_TIME_FORMAT " [%"
           GST_TIME_FORMAT " - %" GST_TIME_FORMAT "]",
           GST_TIME_ARGS (dec->segment.last_stop),
@@ -535,10 +534,10 @@ speex_dec_chain_parse_header (GstSpeexDec * dec, GstBuffer * buf)
   speex_decoder_ctl (dec->state, SPEEX_GET_FRAME_SIZE, &dec->frame_size);
 
   if (dec->header->nb_channels != 1) {
+    dec->stereo = speex_stereo_state_init ();
     dec->callback.callback_id = SPEEX_INBAND_STEREO;
     dec->callback.func = speex_std_stereo_request_handler;
-    dec->callback.data = &dec->stereo;
-    dec->stereo = (SpeexStereoState) SPEEX_STEREO_STATE_INIT;
+    dec->callback.data = dec->stereo;
     speex_decoder_ctl (dec->state, SPEEX_SET_HANDLER, &dec->callback);
   }
 
@@ -647,9 +646,13 @@ speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf,
   guint8 *data;
   SpeexBits *bits;
 
+  if (!dec->frame_duration)
+    goto not_negotiated;
+
   if (timestamp != -1) {
     dec->segment.last_stop = timestamp;
-    dec->granulepos = -1;
+  } else {
+    timestamp = dec->segment.last_stop;
   }
 
   if (buf) {
@@ -663,16 +666,6 @@ speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf,
     bits = &dec->bits;
 
     GST_DEBUG_OBJECT (dec, "received buffer of size %u, fpp %d", size, fpp);
-
-    if (!GST_BUFFER_TIMESTAMP_IS_VALID (buf)
-        && GST_BUFFER_OFFSET_END_IS_VALID (buf)) {
-      dec->granulepos = GST_BUFFER_OFFSET_END (buf);
-      GST_DEBUG_OBJECT (dec,
-          "Taking granulepos from upstream: %" G_GUINT64_FORMAT,
-          dec->granulepos);
-    }
-
-    /* copy timestamp */
   } else {
     /* concealment data, pass NULL as the bits parameters */
     GST_DEBUG_OBJECT (dec, "creating concealment data");
@@ -722,31 +715,13 @@ speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf,
       break;
     }
     if (dec->header->nb_channels == 2)
-      speex_decode_stereo_int (out_data, dec->frame_size, &dec->stereo);
+      speex_decode_stereo_int (out_data, dec->frame_size, dec->stereo);
 
-    if (dec->granulepos == -1) {
-      if (dec->segment.format != GST_FORMAT_TIME) {
-        GST_WARNING_OBJECT (dec, "segment not initialized or not TIME format");
-        dec->granulepos = dec->frame_size;
-      } else {
-        dec->granulepos = gst_util_uint64_scale_int (dec->segment.last_stop,
-            dec->header->rate, GST_SECOND) + dec->frame_size;
-      }
-      GST_DEBUG_OBJECT (dec, "granulepos=%" G_GINT64_FORMAT, dec->granulepos);
-    }
-
-    if (timestamp == -1) {
-      timestamp = gst_util_uint64_scale_int (dec->granulepos - dec->frame_size,
-          GST_SECOND, dec->header->rate);
-    }
-
-    GST_BUFFER_OFFSET (outbuf) = dec->granulepos - dec->frame_size;
-    GST_BUFFER_OFFSET_END (outbuf) = dec->granulepos;
     GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
     GST_BUFFER_DURATION (outbuf) = dec->frame_duration;
 
-    dec->granulepos += dec->frame_size;
     dec->segment.last_stop += dec->frame_duration;
+    timestamp = dec->segment.last_stop;
 
     GST_LOG_OBJECT (dec, "pushing buffer with ts=%" GST_TIME_FORMAT ", dur=%"
         GST_TIME_FORMAT, GST_TIME_ARGS (timestamp),
@@ -758,10 +733,17 @@ speex_dec_chain_parse_data (GstSpeexDec * dec, GstBuffer * buf,
       GST_DEBUG_OBJECT (dec, "flow: %s", gst_flow_get_name (res));
       break;
     }
-    timestamp = -1;
   }
 
   return res;
+
+  /* ERRORS */
+not_negotiated:
+  {
+    GST_ELEMENT_ERROR (dec, CORE, NEGOTIATION, (NULL),
+        ("decoder not initialized"));
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
 }
 
 static GstFlowReturn
@@ -780,10 +762,12 @@ speex_dec_chain (GstPad * pad, GstBuffer * buf)
       res = speex_dec_chain_parse_comments (dec, buf);
       break;
     default:
+    {
       res =
           speex_dec_chain_parse_data (dec, buf, GST_BUFFER_TIMESTAMP (buf),
           GST_BUFFER_DURATION (buf));
       break;
+    }
   }
 
   dec->packetno++;
