@@ -79,12 +79,6 @@ enum
 
 #define DEFAULT_BIGFILE TRUE
 
-static const GstElementDetails gst_avi_mux_details =
-GST_ELEMENT_DETAILS ("Avi muxer",
-    "Codec/Muxer",
-    "Muxes audio and video into an avi stream",
-    "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
-
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -154,6 +148,10 @@ static GstStaticPadTemplate video_sink_factory =
         "image/x-jpc, "
         "width = (int) [ 1, 2147483647 ], "
         "height = (int) [ 1, 2147483647 ], "
+        "framerate = (fraction) [ 0, MAX ];"
+        "video/x-vp8, "
+        "width = (int) [ 1, 2147483647 ], "
+        "height = (int) [ 1, 2147483647 ], "
         "framerate = (fraction) [ 0, MAX ]")
     );
 
@@ -254,7 +252,10 @@ gst_avi_mux_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&video_sink_factory));
 
-  gst_element_class_set_details (element_class, &gst_avi_mux_details);
+  gst_element_class_set_details_simple (element_class, "Avi muxer",
+      "Codec/Muxer",
+      "Muxes audio and video into an avi stream",
+      "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
 
   GST_DEBUG_CATEGORY_INIT (avimux_debug, "avimux", 0, "Muxer for AVI streams");
 }
@@ -338,6 +339,11 @@ gst_avi_mux_pad_reset (GstAviPad * avipad, gboolean free)
       vidpad->vids_codec_data = NULL;
     }
 
+    if (vidpad->prepend_buffer) {
+      gst_buffer_unref (vidpad->prepend_buffer);
+      vidpad->prepend_buffer = NULL;
+    }
+
     memset (&(vidpad->vids), 0, sizeof (gst_riff_strf_vids));
     memset (&(vidpad->vprp), 0, sizeof (gst_riff_vprp));
   } else {
@@ -433,6 +439,7 @@ gst_avi_mux_vidsink_set_caps (GstPad * pad, GstCaps * vscaps)
   const GValue *codec_data;
   gint width, height;
   gint par_n, par_d;
+  gboolean codec_data_in_headers = TRUE;
 
   avimux = GST_AVI_MUX (gst_pad_get_parent (pad));
 
@@ -498,15 +505,6 @@ gst_avi_mux_vidsink_set_caps (GstPad * pad, GstCaps * vscaps)
     avipad->vprp.field_info[0].valid_bm_width = width;
   }
 
-  /* codec initialization data, if any */
-  codec_data = gst_structure_get_value (structure, "codec_data");
-  if (codec_data) {
-    avipad->vids_codec_data = gst_value_get_buffer (codec_data);
-    gst_buffer_ref (avipad->vids_codec_data);
-    /* keep global track of size */
-    avimux->codec_data_size += GST_BUFFER_SIZE (avipad->vids_codec_data);
-  }
-
   if (!strcmp (mimetype, "video/x-raw-yuv")) {
     guint32 format;
 
@@ -586,6 +584,11 @@ gst_avi_mux_vidsink_set_caps (GstPad * pad, GstCaps * vscaps)
         case 4:
           /* mplayer/ffmpeg might not work with DIVX, but with FMP4 */
           avipad->vids.compression = GST_MAKE_FOURCC ('D', 'I', 'V', 'X');
+
+          /* DIVX/XVID in AVI store the codec_data chunk as part of the
+             first data buffer. So for this case, we prepend the codec_data
+             blob (if any) to that first buffer */
+          codec_data_in_headers = FALSE;
           break;
         default:
           GST_INFO ("unhandled mpegversion : %d, fall back to fourcc=MPEG",
@@ -614,12 +617,28 @@ gst_avi_mux_vidsink_set_caps (GstPad * pad, GstCaps * vscaps)
       }
     } else if (!strcmp (mimetype, "image/x-jpc")) {
       avipad->vids.compression = GST_MAKE_FOURCC ('M', 'J', '2', 'C');
+    } else if (!strcmp (mimetype, "video/x-vp8")) {
+      avipad->vids.compression = GST_MAKE_FOURCC ('V', 'P', '8', '0');
     }
 
     if (!avipad->vids.compression)
       goto refuse_caps;
   }
 
+  /* codec initialization data, if any */
+  codec_data = gst_structure_get_value (structure, "codec_data");
+  if (codec_data) {
+    if (codec_data_in_headers) {
+      avipad->vids_codec_data = gst_value_get_buffer (codec_data);
+      gst_buffer_ref (avipad->vids_codec_data);
+      /* keep global track of size */
+      avimux->codec_data_size += GST_BUFFER_SIZE (avipad->vids_codec_data);
+    } else {
+      avipad->prepend_buffer =
+          gst_buffer_ref (gst_value_get_buffer (codec_data));
+    }
+  }
+
   avipad->parent.hdr.fcc_handler = avipad->vids.compression;
   avipad->vids.image_size = avipad->vids.height * avipad->vids.width;
   /* hm, maybe why avi only handles one stream well ... */
@@ -921,6 +940,7 @@ gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
   if (!avipad->auds.format)
     goto refuse_caps;
 
+  avipad->parent.hdr.fcc_handler = avipad->auds.format;
   gst_avi_mux_audsink_set_fields (avimux, avipad);
 
   gst_object_unref (avimux);
@@ -1080,7 +1100,7 @@ gst_avi_mux_write_tag (const GstTagList * list, const gchar * tag,
   const struct
   {
     guint32 fcc;
-    gchar *tag;
+    const gchar *tag;
   } rifftags[] = {
     {
     GST_RIFF_INFO_IARL, GST_TAG_LOCATION}, {
@@ -1146,7 +1166,10 @@ gst_avi_mux_riff_get_avi_header (GstAviMux * avimux)
   else {
     /* need to make snapshot of current state of tags to ensure the same set
      * is used next time around during header rewrite at the end */
+    /* FIXME: remove locking again after GstTagSetter has been fixed */
+    GST_OBJECT_LOCK (avimux);
     tags = gst_tag_setter_get_tag_list (GST_TAG_SETTER (avimux));
+    GST_OBJECT_UNLOCK (avimux);
     if (tags)
       tags = avimux->tags_snap = gst_tag_list_copy (tags);
   }
@@ -1899,10 +1922,15 @@ gst_avi_mux_handle_event (GstPad * pad, GstEvent * event)
     case GST_EVENT_TAG:{
       GstTagList *list;
       GstTagSetter *setter = GST_TAG_SETTER (avimux);
-      const GstTagMergeMode mode = gst_tag_setter_get_tag_merge_mode (setter);
+      GstTagMergeMode mode;
 
       gst_event_parse_tag (event, &list);
+
+      /* FIXME: remove locking again after GstTagSetter has been fixed */
+      GST_OBJECT_LOCK (avimux);
+      mode = gst_tag_setter_get_tag_merge_mode (setter);
       gst_tag_setter_merge_tags (setter, list, mode);
+      GST_OBJECT_UNLOCK (avimux);
       break;
     }
     default:
@@ -1940,6 +1968,21 @@ gst_avi_mux_do_buffer (GstAviMux * avimux, GstAviPad * avipad)
 
   data = gst_collect_pads_pop (avimux->collect, avipad->collect);
 
+  /* Prepend a special buffer to the first one for some formats */
+  if (avipad->is_video) {
+    GstAviVideoPad *vidpad = (GstAviVideoPad *) avipad;
+
+    if (vidpad->prepend_buffer) {
+      GstBuffer *newdata = gst_buffer_merge (vidpad->prepend_buffer, data);
+      gst_buffer_copy_metadata (newdata, data, GST_BUFFER_COPY_TIMESTAMPS);
+      gst_buffer_unref (data);
+      gst_buffer_unref (vidpad->prepend_buffer);
+
+      data = newdata;
+      vidpad->prepend_buffer = NULL;
+    }
+  }
+
   if (avimux->restart) {
     if ((res = gst_avi_mux_restart_file (avimux)) != GST_FLOW_OK)
       return res;
@@ -2041,6 +2084,9 @@ gst_avi_mux_do_one_buffer (GstAviMux * avimux)
     if (!avipad->collect)
       continue;
 
+    if (!avipad->hdr.fcc_handler)
+      goto not_negotiated;
+
     buffer = gst_collect_pads_peek (avimux->collect, avipad->collect);
     if (!buffer)
       continue;
@@ -2069,6 +2115,14 @@ gst_avi_mux_do_one_buffer (GstAviMux * avimux)
     gst_pad_push_event (avimux->srcpad, gst_event_new_eos ());
     return GST_FLOW_UNEXPECTED;
   }
+
+  /* ERRORS */
+not_negotiated:
+  {
+    GST_ELEMENT_ERROR (avimux, CORE, NEGOTIATION, (NULL),
+        ("pad %s not negotiated", GST_PAD_NAME (avipad->collect->pad)));
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
 }
 
 static GstFlowReturn
