@@ -24,7 +24,6 @@
 #endif
 
 #include <string.h>
-#include <gst/floatcast/floatcast.h>
 
 #include "ebml-read.h"
 #include "ebml-ids.h"
@@ -112,7 +111,6 @@ static void
 gst_ebml_read_class_init (GstEbmlReadClass * klass)
 {
   GstElementClass *gstelement_class = (GstElementClass *) klass;
-
   GObjectClass *gobject_class = (GObjectClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
@@ -137,7 +135,6 @@ static GstStateChangeReturn
 gst_ebml_read_change_state (GstElement * element, GstStateChange transition)
 {
   GstStateChangeReturn ret;
-
   GstEbmlRead *ebml = GST_EBML_READ (element);
 
   switch (transition) {
@@ -172,6 +169,29 @@ gst_ebml_read_change_state (GstElement * element, GstStateChange transition)
   return ret;
 }
 
+/*
+ * Used in push mode.
+ * Provided buffer is used as cache, based on offset 0, and no further reads
+ * will be issued.
+ */
+
+void
+gst_ebml_read_reset_cache (GstEbmlRead * ebml, GstBuffer * buffer,
+    guint64 offset)
+{
+  if (ebml->cached_buffer)
+    gst_buffer_unref (ebml->cached_buffer);
+
+  ebml->cached_buffer = buffer;
+  ebml->push_cache = TRUE;
+  buffer = gst_buffer_make_metadata_writable (buffer);
+  GST_BUFFER_OFFSET (buffer) = offset;
+  ebml->offset = offset;
+  g_list_foreach (ebml->level, (GFunc) gst_ebml_level_free, NULL);
+  g_list_free (ebml->level);
+  ebml->level = NULL;
+}
+
 /*
  * Return: the amount of levels in the hierarchy that the
  * current element lies higher than the previous one.
@@ -183,7 +203,6 @@ static guint
 gst_ebml_read_element_level_up (GstEbmlRead * ebml)
 {
   guint num = 0;
-
   guint64 pos = ebml->offset;
 
   while (ebml->level != NULL) {
@@ -214,11 +233,10 @@ gst_ebml_read_peek_bytes (GstEbmlRead * ebml, guint size, GstBuffer ** p_buf,
    * We do it mainly to avoid pulling buffers of 1 byte all the time */
   if (ebml->cached_buffer) {
     guint64 cache_offset = GST_BUFFER_OFFSET (ebml->cached_buffer);
-
     guint cache_size = GST_BUFFER_SIZE (ebml->cached_buffer);
 
     if (cache_offset <= ebml->offset &&
-        (ebml->offset + size) < (cache_offset + cache_size)) {
+        (ebml->offset + size) <= (cache_offset + cache_size)) {
       if (p_buf)
         *p_buf = gst_buffer_create_sub (ebml->cached_buffer,
             ebml->offset - cache_offset, size);
@@ -228,6 +246,13 @@ gst_ebml_read_peek_bytes (GstEbmlRead * ebml, guint size, GstBuffer ** p_buf,
       return GST_FLOW_OK;
     }
     /* not enough data in the cache, free cache and get a new one */
+    /* never drop pushed cache */
+    if (ebml->push_cache) {
+      if (ebml->offset == cache_offset + cache_size)
+        return GST_FLOW_END;
+      else
+        return GST_FLOW_UNEXPECTED;
+    }
     gst_buffer_unref (ebml->cached_buffer);
     ebml->cached_buffer = NULL;
   }
@@ -313,13 +338,9 @@ static GstFlowReturn
 gst_ebml_read_element_id (GstEbmlRead * ebml, guint32 * id, guint * level_up)
 {
   guint8 *buf;
-
   gint len_mask = 0x80, read = 1, n = 1;
-
   guint32 total;
-
   guint8 b;
-
   GstFlowReturn ret;
 
   ret = gst_ebml_read_peek_bytes (ebml, 1, NULL, &buf);
@@ -335,11 +356,9 @@ gst_ebml_read_element_id (GstEbmlRead * ebml, guint32 * id, guint * level_up)
     len_mask >>= 1;
   }
   if (read > 4) {
-    guint64 pos = ebml->offset;
-
-    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-        ("Invalid EBML ID size tag (0x%x) at position %" G_GUINT64_FORMAT
-            " (0x%" G_GINT64_MODIFIER "x)", (guint) b, pos, pos));
+    GST_ERROR_OBJECT (ebml,
+        "Invalid EBML ID size tag (0x%x) at position %" G_GUINT64_FORMAT " (0x%"
+        G_GINT64_MODIFIER "x)", (guint) b, ebml->offset, ebml->offset);
     return GST_FLOW_ERROR;
   }
 
@@ -373,13 +392,9 @@ gst_ebml_read_element_length (GstEbmlRead * ebml, guint64 * length,
     gint * rread)
 {
   GstFlowReturn ret;
-
   guint8 *buf;
-
   gint len_mask = 0x80, read = 1, n = 1, num_ffs = 0;
-
   guint64 total;
-
   guint8 b;
 
   ret = gst_ebml_read_peek_bytes (ebml, 1, NULL, &buf);
@@ -395,11 +410,9 @@ gst_ebml_read_element_length (GstEbmlRead * ebml, guint64 * length,
     len_mask >>= 1;
   }
   if (read > 8) {
-    guint64 pos = ebml->offset;
-
-    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-        ("Invalid EBML length size tag (0x%x) at position %" G_GUINT64_FORMAT
-            " (0x%" G_GINT64_MODIFIER "x)", (guint) b, pos, pos));
+    GST_ERROR_OBJECT (ebml,
+        "Invalid EBML length size tag (0x%x) at position %" G_GUINT64_FORMAT
+        " (0x%" G_GINT64_MODIFIER "x)", (guint) b, ebml->offset, ebml->offset);
     return GST_FLOW_ERROR;
   }
 
@@ -442,9 +455,7 @@ GstFlowReturn
 gst_ebml_peek_id (GstEbmlRead * ebml, guint * level_up, guint32 * id)
 {
   guint64 off;
-
   guint level_up_tmp = 0;
-
   GstFlowReturn ret;
 
   g_assert (level_up);
@@ -455,8 +466,17 @@ gst_ebml_peek_id (GstEbmlRead * ebml, guint * level_up, guint32 * id)
 next:
   off = ebml->offset;           /* save offset */
 
-  if ((ret = gst_ebml_read_element_id (ebml, id, &level_up_tmp)) != GST_FLOW_OK)
-    return ret;
+  if ((ret = gst_ebml_read_element_id (ebml, id, &level_up_tmp)) != GST_FLOW_OK) {
+    if (ret != GST_FLOW_END)
+      return ret;
+    else {
+      /* simulate dummy VOID element,
+       * and have the call stack bail out all the way */
+      *id = GST_EBML_ID_VOID;
+      *level_up = G_MAXUINT32 >> 2;
+      return GST_FLOW_OK;
+    }
+  }
 
   ebml->offset = off;           /* restore offset */
 
@@ -489,7 +509,6 @@ gint64
 gst_ebml_read_get_length (GstEbmlRead * ebml)
 {
   GstFormat fmt = GST_FORMAT_BYTES;
-
   gint64 end;
 
   /* FIXME: what to do if we don't get the upstream length */
@@ -523,9 +542,7 @@ GstFlowReturn
 gst_ebml_read_skip (GstEbmlRead * ebml)
 {
   guint64 length;
-
   guint32 id;
-
   GstFlowReturn ret;
 
   ret = gst_ebml_read_element_id (ebml, &id, NULL);
@@ -548,7 +565,6 @@ GstFlowReturn
 gst_ebml_read_buffer (GstEbmlRead * ebml, guint32 * id, GstBuffer ** buf)
 {
   guint64 length;
-
   GstFlowReturn ret;
 
   ret = gst_ebml_read_element_id (ebml, id, NULL);
@@ -579,7 +595,6 @@ gst_ebml_read_bytes (GstEbmlRead * ebml, guint32 * id, guint8 ** data,
     guint * size)
 {
   guint64 length;
-
   GstFlowReturn ret;
 
   *size = 0;
@@ -615,9 +630,7 @@ GstFlowReturn
 gst_ebml_read_uint (GstEbmlRead * ebml, guint32 * id, guint64 * num)
 {
   guint8 *data;
-
   guint size;
-
   GstFlowReturn ret;
 
   ret = gst_ebml_read_bytes (ebml, id, &data, &size);
@@ -625,10 +638,9 @@ gst_ebml_read_uint (GstEbmlRead * ebml, guint32 * id, guint64 * num)
     return ret;
 
   if (size < 1 || size > 8) {
-    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-        ("Invalid integer element size %d at position %" G_GUINT64_FORMAT
-            " (0x%" G_GINT64_MODIFIER "x)",
-            size, ebml->offset - size, ebml->offset - size));
+    GST_ERROR_OBJECT (ebml,
+        "Invalid integer element size %d at position %" G_GUINT64_FORMAT " (0x%"
+        G_GINT64_MODIFIER "x)", size, ebml->offset - size, ebml->offset - size);
     return GST_FLOW_ERROR;
   }
   *num = 0;
@@ -649,11 +661,8 @@ GstFlowReturn
 gst_ebml_read_sint (GstEbmlRead * ebml, guint32 * id, gint64 * num)
 {
   guint8 *data;
-
   guint size;
-
   gboolean negative = 0;
-
   GstFlowReturn ret;
 
   ret = gst_ebml_read_bytes (ebml, id, &data, &size);
@@ -661,10 +670,9 @@ gst_ebml_read_sint (GstEbmlRead * ebml, guint32 * id, gint64 * num)
     return ret;
 
   if (size < 1 || size > 8) {
-    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-        ("Invalid integer element size %d at position %" G_GUINT64_FORMAT
-            " (0x%" G_GINT64_MODIFIER "x)", size, ebml->offset - size,
-            ebml->offset - size));
+    GST_ERROR_OBJECT (ebml,
+        "Invalid integer element size %d at position %" G_GUINT64_FORMAT " (0x%"
+        G_GINT64_MODIFIER "x)", size, ebml->offset - size, ebml->offset - size);
     return GST_FLOW_ERROR;
   }
 
@@ -704,9 +712,7 @@ static gdouble
 _ext2dbl (guint8 * data)
 {
   struct _ext_float ext;
-
   guint64 m = 0;
-
   gint e, i;
 
   memcpy (&ext.exponent, data, 2);
@@ -733,9 +739,7 @@ GstFlowReturn
 gst_ebml_read_float (GstEbmlRead * ebml, guint32 * id, gdouble * num)
 {
   guint8 *data;
-
   guint size;
-
   GstFlowReturn ret;
 
   ret = gst_ebml_read_bytes (ebml, id, &data, &size);
@@ -743,10 +747,9 @@ gst_ebml_read_float (GstEbmlRead * ebml, guint32 * id, gdouble * num)
     return ret;
 
   if (size != 4 && size != 8 && size != 10) {
-    GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-        ("Invalid float element size %d at position %" G_GUINT64_FORMAT
-            " (0x%" G_GINT64_MODIFIER "x)", size, ebml->offset - size,
-            ebml->offset - size));
+    GST_ERROR_OBJECT (ebml,
+        "Invalid float element size %d at position %" G_GUINT64_FORMAT " (0x%"
+        G_GINT64_MODIFIER "x)", size, ebml->offset - size, ebml->offset - size);
     return GST_FLOW_ERROR;
   }
 
@@ -772,16 +775,14 @@ gst_ebml_read_float (GstEbmlRead * ebml, guint32 * id, gdouble * num)
 }
 
 /*
- * Read the next element as an ASCII string.
+ * Read the next element as a C string.
  */
 
-GstFlowReturn
-gst_ebml_read_ascii (GstEbmlRead * ebml, guint32 * id, gchar ** str)
+static GstFlowReturn
+gst_ebml_read_string (GstEbmlRead * ebml, guint32 * id, gchar ** str)
 {
   guint8 *data;
-
   guint size;
-
   GstFlowReturn ret;
 
   ret = gst_ebml_read_bytes (ebml, id, &data, &size);
@@ -795,6 +796,38 @@ gst_ebml_read_ascii (GstEbmlRead * ebml, guint32 * id, gchar ** str)
   return ret;
 }
 
+/*
+ * Read the next element as an ASCII string.
+ */
+
+GstFlowReturn
+gst_ebml_read_ascii (GstEbmlRead * ebml, guint32 * id, gchar ** str_out)
+{
+  GstFlowReturn ret;
+  gchar *str;
+  gchar *iter;
+
+#ifndef GST_DISABLE_GST_DEBUG
+  guint64 oldoff = ebml->offset;
+#endif
+
+  ret = gst_ebml_read_string (ebml, id, &str);
+  if (ret != GST_FLOW_OK)
+    return ret;
+
+  for (iter = str; *iter != '\0'; iter++) {
+    if (G_UNLIKELY (*iter & 0x80)) {
+      GST_ERROR_OBJECT (ebml,
+          "Invalid ASCII string at offset %" G_GUINT64_FORMAT, oldoff);
+      g_free (str);
+      return GST_FLOW_ERROR;
+    }
+  }
+
+  *str_out = str;
+  return ret;
+}
+
 /*
  * Read the next element as a UTF-8 string.
  */
@@ -808,7 +841,7 @@ gst_ebml_read_utf8 (GstEbmlRead * ebml, guint32 * id, gchar ** str)
   guint64 oldoff = ebml->offset;
 #endif
 
-  ret = gst_ebml_read_ascii (ebml, id, str);
+  ret = gst_ebml_read_string (ebml, id, str);
   if (ret != GST_FLOW_OK)
     return ret;
 
@@ -830,7 +863,6 @@ GstFlowReturn
 gst_ebml_read_date (GstEbmlRead * ebml, guint32 * id, gint64 * date)
 {
   gint64 ebml_date;
-
   GstFlowReturn ret;
 
   ret = gst_ebml_read_sint (ebml, id, &ebml_date);
@@ -851,9 +883,7 @@ GstFlowReturn
 gst_ebml_read_master (GstEbmlRead * ebml, guint32 * id)
 {
   GstEbmlLevel *level;
-
   guint64 length;
-
   GstFlowReturn ret;
 
   ret = gst_ebml_read_element_id (ebml, id, NULL);
@@ -882,9 +912,7 @@ gst_ebml_read_binary (GstEbmlRead * ebml,
     guint32 * id, guint8 ** binary, guint64 * length)
 {
   guint8 *data;
-
   guint size;
-
   GstFlowReturn ret;
 
   ret = gst_ebml_read_bytes (ebml, id, &data, &size);
@@ -906,9 +934,7 @@ gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
 {
   /* this function is the first to be called */
   guint32 id;
-
   guint level_up;
-
   GstFlowReturn ret;
 
   /* default init */
@@ -924,7 +950,7 @@ gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
   GST_DEBUG_OBJECT (ebml, "id: %08x", GST_READ_UINT32_BE (&id));
 
   if (level_up != 0 || id != GST_EBML_ID_HEADER) {
-    GST_ELEMENT_ERROR (ebml, STREAM, WRONG_TYPE, (NULL), (NULL));
+    GST_ERROR_OBJECT (ebml, "Failed to read header");
     return GST_FLOW_ERROR;
   }
   ret = gst_ebml_read_master (ebml, &id);
@@ -950,7 +976,8 @@ gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
           return ret;
         g_assert (id == GST_EBML_ID_EBMLREADVERSION);
         if (num != GST_EBML_VERSION) {
-          GST_ELEMENT_ERROR (ebml, STREAM, WRONG_TYPE, (NULL), (NULL));
+          GST_ERROR_OBJECT (ebml, "Unsupported EBML version %" G_GUINT64_FORMAT,
+              num);
           return GST_FLOW_ERROR;
         }
 
@@ -967,7 +994,8 @@ gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
           return ret;
         g_assert (id == GST_EBML_ID_EBMLMAXSIZELENGTH);
         if (num > sizeof (guint64)) {
-          GST_ELEMENT_ERROR (ebml, STREAM, WRONG_TYPE, (NULL), (NULL));
+          GST_ERROR_OBJECT (ebml,
+              "Unsupported EBML maximum size %" G_GUINT64_FORMAT, num);
           return GST_FLOW_ERROR;
         }
         GST_DEBUG_OBJECT (ebml, "EbmlMaxSizeLength: %" G_GUINT64_FORMAT, num);
@@ -983,7 +1011,8 @@ gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
           return ret;
         g_assert (id == GST_EBML_ID_EBMLMAXIDLENGTH);
         if (num > sizeof (guint32)) {
-          GST_ELEMENT_ERROR (ebml, STREAM, WRONG_TYPE, (NULL), (NULL));
+          GST_ERROR_OBJECT (ebml,
+              "Unsupported EBML maximum ID %" G_GUINT64_FORMAT, num);
           return GST_FLOW_ERROR;
         }
         GST_DEBUG_OBJECT (ebml, "EbmlMaxIdLength: %" G_GUINT64_FORMAT, num);
