@@ -85,8 +85,11 @@ typedef struct _GstMatroskaMux {
   /* Application name (for the writing application header element) */
   gchar          *writing_app;
 
-  /* Matroska version. */
-  guint          matroska_version;
+  /* EBML DocType. */
+  const gchar    *doctype;
+
+  /* DocType version. */
+  guint          doctype_version;
 
   /* state */
   GstMatroskaMuxState state;
@@ -116,7 +119,8 @@ typedef struct _GstMatroskaMux {
   /* current cluster */
   guint64        cluster,
                  cluster_time,
-                 cluster_pos;
+                 cluster_pos,
+		 prev_cluster_size;
 
 } GstMatroskaMux;
 
@@ -124,7 +128,7 @@ typedef struct _GstMatroskaMuxClass {
   GstElementClass parent;
 } GstMatroskaMuxClass;
 
-gboolean gst_matroska_mux_plugin_init (GstPlugin *plugin);
+GType   gst_matroska_mux_get_type (void);
 
 G_END_DECLS
 
