@@ -48,13 +48,6 @@ gst_rtp_g729_pay_set_caps (GstBaseRTPPayload * payload, GstCaps * caps);
 static GstFlowReturn
 gst_rtp_g729_pay_handle_buffer (GstBaseRTPPayload * payload, GstBuffer * buf);
 
-
-static const GstElementDetails gst_rtp_g729_pay_details =
-GST_ELEMENT_DETAILS ("RTP G.729 payloader",
-    "Codec/Payloader/Network",
-    "Packetize G.729 audio into RTP packets",
-    "Olivier Crete <olivier.crete@collabora.co.uk>");
-
 static GstStaticPadTemplate gst_rtp_g729_pay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -78,9 +71,6 @@ static GstStaticPadTemplate gst_rtp_g729_pay_src_template =
         "clock-rate = (int) 8000, " "encoding-name = (string) \"G729\"")
     );
 
-static void
-gst_rtp_g729_pay_init (GstRTPG729Pay * pay, GstRTPG729PayClass * klass);
-
 GST_BOILERPLATE (GstRTPG729Pay, gst_rtp_g729_pay, GstBaseRTPAudioPayload,
     GST_TYPE_BASE_RTP_AUDIO_PAYLOAD);
 
@@ -93,7 +83,10 @@ gst_rtp_g729_pay_base_init (gpointer klass)
       gst_static_pad_template_get (&gst_rtp_g729_pay_sink_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_g729_pay_src_template));
-  gst_element_class_set_details (element_class, &gst_rtp_g729_pay_details);
+  gst_element_class_set_details_simple (element_class, "RTP G.729 payloader",
+      "Codec/Payloader/Network",
+      "Packetize G.729 audio into RTP packets",
+      "Olivier Crete <olivier.crete@collabora.co.uk>");
 
   GST_DEBUG_CATEGORY_INIT (rtpg729pay_debug, "rtpg729pay", 0,
       "G.729 RTP Payloader");
@@ -150,13 +143,11 @@ gst_rtp_g729_pay_handle_buffer (GstBaseRTPPayload * payload, GstBuffer * buf)
       GST_BASE_RTP_AUDIO_PAYLOAD (payload);
   GstAdapter *adapter = NULL;
   guint payload_len;
-  const guint8 *data = NULL;
   guint available;
   guint maxptime_octets = G_MAXUINT;
   guint minptime_octets = 0;
   guint min_payload_len;
   guint max_payload_len;
-  gboolean use_adapter = FALSE;
 
   available = GST_BUFFER_SIZE (buf);
 
@@ -167,7 +158,7 @@ gst_rtp_g729_pay_handle_buffer (GstBaseRTPPayload * payload, GstBuffer * buf)
   /* max number of bytes based on given ptime, has to be multiple of
    * frame_duration */
   if (payload->max_ptime != -1) {
-    guint ptime_ms = payload->max_ptime / 1000000;
+    guint ptime_ms = payload->max_ptime / GST_MSECOND;
 
     maxptime_octets = G729_FRAME_SIZE *
         (int) (ptime_ms / G729_FRAME_DURATION_MS);
@@ -190,11 +181,9 @@ gst_rtp_g729_pay_handle_buffer (GstBaseRTPPayload * payload, GstBuffer * buf)
   /* min number of bytes based on a given ptime, has to be a multiple
      of frame duration */
   {
-    guint64 min_ptime;
+    guint64 min_ptime = payload->min_ptime;
 
-    g_object_get (G_OBJECT (payload), "min-ptime", &min_ptime, NULL);
-
-    min_ptime = min_ptime / 1000000;
+    min_ptime = min_ptime / GST_MSECOND;
     minptime_octets = G729_FRAME_SIZE *
         (int) (min_ptime / G729_FRAME_DURATION_MS);
   }
@@ -205,37 +194,44 @@ gst_rtp_g729_pay_handle_buffer (GstBaseRTPPayload * payload, GstBuffer * buf)
     min_payload_len = max_payload_len;
   }
 
+  /* If the ptime is specified in the caps, tried to adhere to it exactly */
+  if (payload->abidata.ABI.ptime) {
+    guint64 ptime = payload->abidata.ABI.ptime / GST_MSECOND;
+    guint ptime_in_bytes = G729_FRAME_SIZE *
+        (guint) (ptime / G729_FRAME_DURATION_MS);
+
+    /* clip to computed min and max lengths */
+    ptime_in_bytes = MAX (min_payload_len, ptime_in_bytes);
+    ptime_in_bytes = MIN (max_payload_len, ptime_in_bytes);
+
+    min_payload_len = max_payload_len = ptime_in_bytes;
+  }
+
   GST_LOG_OBJECT (basertpaudiopayload,
       "Calculated min_payload_len %u and max_payload_len %u",
       min_payload_len, max_payload_len);
 
   adapter = gst_base_rtp_audio_payload_get_adapter (basertpaudiopayload);
 
-  if (adapter && gst_adapter_available (adapter)) {
-    /* If there is always data in the adapter, we have to use it */
-    gst_adapter_push (adapter, buf);
-    available = gst_adapter_available (adapter);
-    use_adapter = TRUE;
-  } else {
-    /* let's set the base timestamp */
+
+  /* let's reset the base timestamp when the adapter is empty */
+  if (gst_adapter_available (adapter) == 0)
     basertpaudiopayload->base_ts = GST_BUFFER_TIMESTAMP (buf);
 
-    /* If buffer fits on an RTP packet, let's just push it through */
-    /* this will check against max_ptime and max_mtu */
-    if (GST_BUFFER_SIZE (buf) >= min_payload_len &&
-        GST_BUFFER_SIZE (buf) <= max_payload_len) {
-      ret = gst_base_rtp_audio_payload_push (basertpaudiopayload,
-          GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf),
-          GST_BUFFER_TIMESTAMP (buf));
-      gst_buffer_unref (buf);
-      g_object_unref (adapter);
-      return ret;
-    }
-
-    available = GST_BUFFER_SIZE (buf);
-    data = (guint8 *) GST_BUFFER_DATA (buf);
+  if (gst_adapter_available (adapter) == 0 &&
+      GST_BUFFER_SIZE (buf) >= min_payload_len &&
+      GST_BUFFER_SIZE (buf) <= max_payload_len) {
+    ret = gst_base_rtp_audio_payload_push (basertpaudiopayload,
+        GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf),
+        GST_BUFFER_TIMESTAMP (buf));
+    gst_buffer_unref (buf);
+    g_object_unref (adapter);
+    return ret;
   }
 
+  gst_adapter_push (adapter, buf);
+
+  available = gst_adapter_available (adapter);
   /* as long as we have full frames */
   /* this loop will push all available buffers till the last frame */
   while (available >= min_payload_len ||
@@ -250,38 +246,16 @@ gst_rtp_g729_pay_handle_buffer (GstBaseRTPPayload * payload, GstBuffer * buf)
           (available / G729_FRAME_SIZE) * G729_FRAME_SIZE);
     }
 
-    if (use_adapter) {
-      data = gst_adapter_peek (adapter, payload_len);
-    }
-
-    ret = gst_base_rtp_audio_payload_push (basertpaudiopayload, data,
-        payload_len, basertpaudiopayload->base_ts);
+    ret = gst_base_rtp_audio_payload_flush (basertpaudiopayload, payload_len,
+        basertpaudiopayload->base_ts);
 
     num = payload_len / G729_FRAME_SIZE;
     basertpaudiopayload->base_ts += G729_FRAME_DURATION * num;
 
-    if (use_adapter) {
-      gst_adapter_flush (adapter, payload_len);
-      available = gst_adapter_available (adapter);
-    } else {
-      available -= payload_len;
-      data += payload_len;
-    }
+    available = gst_adapter_available (adapter);
   }
 
-  if (!use_adapter) {
-    if (available != 0 && adapter) {
-      GstBuffer *buf2;
-      buf2 = gst_buffer_create_sub (buf,
-          GST_BUFFER_SIZE (buf) - available, available);
-      gst_adapter_push (adapter, buf2);
-    }
-    gst_buffer_unref (buf);
-  }
-
-  if (adapter) {
-    g_object_unref (adapter);
-  }
+  g_object_unref (adapter);
 
   return ret;
 
