@@ -26,13 +26,6 @@
 #include <gst/rtp/gstrtpbuffer.h>
 #include "gstrtph263pdepay.h"
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_h263pdepay_details =
-GST_ELEMENT_DETAILS ("RTP H263 depayloader",
-    "Codec/Depayloader/Network",
-    "Extracts H263/+/++ video from RTP packets (RFC 4629)",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 static GstStaticPadTemplate gst_rtp_h263p_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -107,7 +100,10 @@ gst_rtp_h263p_depay_base_init (gpointer klass)
       gst_static_pad_template_get (&gst_rtp_h263p_depay_sink_template));
 
 
-  gst_element_class_set_details (element_class, &gst_rtp_h263pdepay_details);
+  gst_element_class_set_details_simple (element_class, "RTP H263 depayloader",
+      "Codec/Depayloader/Network",
+      "Extracts H263/+/++ video from RTP packets (RFC 4629)",
+      "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
