@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2006> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2006> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -44,9 +44,19 @@ struct _GstRtpH264Depay
 {
   GstBaseRTPDepayload depayload;
 
+  gboolean    byte_stream;
+
   GstBuffer  *codec_data;
   GstAdapter *adapter;
   gboolean    wait_start;
+
+  /* nal merging */
+  gboolean    merge;
+  GstAdapter *picture_adapter;
+  gboolean    picture_start;
+  gboolean    picture_complete;
+  GstClockTime last_ts;
+  gboolean    last_delta;
 };
 
 struct _GstRtpH264DepayClass
@@ -54,6 +64,8 @@ struct _GstRtpH264DepayClass
   GstBaseRTPDepayloadClass parent_class;
 };
 
+GType gst_rtp_h264_depay_get_type (void);
+
 gboolean gst_rtp_h264_depay_plugin_init (GstPlugin * plugin);
 
 G_END_DECLS
