@@ -42,13 +42,6 @@ GST_DEBUG_CATEGORY_STATIC (rtph264pay_debug);
  * RFC 3984
  */
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_h264pay_details =
-GST_ELEMENT_DETAILS ("RTP H264 payloader",
-    "Codec/Payloader/Network",
-    "Payload-encode H264 video into RTP packets (RFC 3984)",
-    "Laurent Glayal <spglegle@yahoo.fr>");
-
 static GstStaticPadTemplate gst_rtp_h264_pay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -119,6 +112,9 @@ static gboolean gst_rtp_h264_pay_setcaps (GstBaseRTPPayload * basepayload,
     GstCaps * caps);
 static GstFlowReturn gst_rtp_h264_pay_handle_buffer (GstBaseRTPPayload * pad,
     GstBuffer * buffer);
+static gboolean gst_rtp_h264_pay_handle_event (GstPad * pad, GstEvent * event);
+static GstStateChangeReturn gst_basertppayload_change_state (GstElement *
+    element, GstStateChange transition);
 
 GST_BOILERPLATE (GstRtpH264Pay, gst_rtp_h264_pay, GstBaseRTPPayload,
     GST_TYPE_BASE_RTP_PAYLOAD);
@@ -133,16 +129,21 @@ gst_rtp_h264_pay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_h264_pay_sink_template));
 
-  gst_element_class_set_details (element_class, &gst_rtp_h264pay_details);
+  gst_element_class_set_details_simple (element_class, "RTP H264 payloader",
+      "Codec/Payloader/Network",
+      "Payload-encode H264 video into RTP packets (RFC 3984)",
+      "Laurent Glayal <spglegle@yahoo.fr>");
 }
 
 static void
 gst_rtp_h264_pay_class_init (GstRtpH264PayClass * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
   GstBaseRTPPayloadClass *gstbasertppayload_class;
 
   gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
   gobject_class->set_property = gst_rtp_h264_pay_set_property;
@@ -187,8 +188,12 @@ gst_rtp_h264_pay_class_init (GstRtpH264PayClass * klass)
 
   gobject_class->finalize = gst_rtp_h264_pay_finalize;
 
+  gstelement_class->change_state =
+      GST_DEBUG_FUNCPTR (gst_basertppayload_change_state);
+
   gstbasertppayload_class->set_caps = gst_rtp_h264_pay_setcaps;
   gstbasertppayload_class->handle_buffer = gst_rtp_h264_pay_handle_buffer;
+  gstbasertppayload_class->handle_event = gst_rtp_h264_pay_handle_event;
 
   GST_DEBUG_CATEGORY_INIT (rtph264pay_debug, "rtph264pay", 0,
       "H264 RTP Payloader");
@@ -419,6 +424,60 @@ avcc_error:
   }
 }
 
+static void
+gst_rtp_h264_pay_parse_sprop_parameter_sets (GstRtpH264Pay * rtph264pay)
+{
+  const gchar *ps;
+  gchar **params;
+  guint len, num_sps, num_pps;
+  gint i;
+  GstBuffer *buf;
+
+  ps = rtph264pay->sprop_parameter_sets;
+  if (ps == NULL)
+    return;
+
+  gst_rtp_h264_pay_clear_sps_pps (rtph264pay);
+
+  params = g_strsplit (ps, ",", 0);
+  len = g_strv_length (params);
+
+  GST_DEBUG_OBJECT (rtph264pay, "we have %d params", len);
+
+  num_sps = num_pps = 0;
+
+  for (i = 0; params[i]; i++) {
+    gsize nal_len;
+    guint8 *nalp;
+    guint save = 0;
+    gint state = 0;
+
+    nal_len = strlen (params[i]);
+    buf = gst_buffer_new_and_alloc (nal_len);
+    nalp = GST_BUFFER_DATA (buf);
+
+    nal_len = g_base64_decode_step (params[i], nal_len, nalp, &state, &save);
+    GST_BUFFER_SIZE (buf) = nal_len;
+
+    if (!nal_len) {
+      gst_buffer_unref (buf);
+      continue;
+    }
+
+    /* append to the right list */
+    if ((nalp[0] & 0x1f) == 7) {
+      GST_DEBUG_OBJECT (rtph264pay, "adding param %d as SPS %d", i, num_sps);
+      rtph264pay->sps = g_list_append (rtph264pay->sps, buf);
+      num_sps++;
+    } else {
+      GST_DEBUG_OBJECT (rtph264pay, "adding param %d as PPS %d", i, num_pps);
+      rtph264pay->pps = g_list_append (rtph264pay->pps, buf);
+      num_pps++;
+    }
+  }
+  g_strfreev (params);
+}
+
 static guint
 next_start_code (guint8 * data, guint size)
 {
@@ -644,18 +703,24 @@ gst_rtp_h264_pay_payload_nal (GstBaseRTPPayload * basepayload, guint8 * data,
   /* check if we need to emit an SPS/PPS now */
   if (nalType == IDR_TYPE_ID && rtph264pay->spspps_interval > 0) {
     if (rtph264pay->last_spspps != -1) {
-      guint diff;
+      guint64 diff;
+
+      GST_LOG_OBJECT (rtph264pay,
+          "now %" GST_TIME_FORMAT ", last SPS/PPS %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (timestamp), GST_TIME_ARGS (rtph264pay->last_spspps));
 
       /* calculate diff between last SPS/PPS in milliseconds */
       if (timestamp > rtph264pay->last_spspps)
-        diff = GST_TIME_AS_MSECONDS (timestamp - rtph264pay->last_spspps);
+        diff = timestamp - rtph264pay->last_spspps;
       else
         diff = 0;
 
-      GST_DEBUG_OBJECT (rtph264pay, "interval since last SPS/PPS %ums", diff);
+      GST_DEBUG_OBJECT (rtph264pay,
+          "interval since last SPS/PPS %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (diff));
 
       /* bigger than interval, queue SPS/PPS */
-      if (diff >= (rtph264pay->spspps_interval * GST_MSECOND)) {
+      if (GST_TIME_AS_SECONDS (diff) >= rtph264pay->spspps_interval) {
         GST_DEBUG_OBJECT (rtph264pay, "time to send SPS/PPS");
         send_spspps = TRUE;
       }
@@ -666,9 +731,10 @@ gst_rtp_h264_pay_payload_nal (GstBaseRTPPayload * basepayload, guint8 * data,
     }
   }
 
-  if (send_spspps) {
+  if (send_spspps || rtph264pay->send_spspps) {
     /* we need to send SPS/PPS now first. FIXME, don't use the timestamp for
      * checking when we need to send SPS/PPS but convert to running_time first. */
+    rtph264pay->send_spspps = FALSE;
     ret = gst_rtp_h264_pay_send_sps_pps (basepayload, rtph264pay, timestamp);
     if (ret != GST_FLOW_OK)
       return ret;
@@ -951,9 +1017,12 @@ gst_rtp_h264_pay_handle_buffer (GstBaseRTPPayload * basepayload,
                   rtph264pay->sprop_parameter_sets, NULL))
             goto caps_rejected;
 
+          /* parse SPS and PPS from provided parameter set (for insertion) */
+          gst_rtp_h264_pay_parse_sprop_parameter_sets (rtph264pay);
+
           rtph264pay->update_caps = FALSE;
 
-          GST_DEBUG ("outcaps udpate: sprop-parameter-sets=%s",
+          GST_DEBUG ("outcaps update: sprop-parameter-sets=%s",
               rtph264pay->sprop_parameter_sets);
         }
       } else {
@@ -1010,6 +1079,51 @@ caps_rejected:
   return GST_FLOW_NOT_NEGOTIATED;
 }
 
+static gboolean
+gst_rtp_h264_pay_handle_event (GstPad * pad, GstEvent * event)
+{
+  const GstStructure *s;
+  GstRtpH264Pay *rtph264pay =
+      GST_RTP_H264_PAY (gst_pad_get_parent_element (pad));
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_CUSTOM_DOWNSTREAM:
+      s = gst_event_get_structure (event);
+      if (gst_structure_has_name (s, "GstForceKeyUnit")) {
+        gboolean resend_codec_data;
+
+        if (gst_structure_get_boolean (s, "all-headers",
+                &resend_codec_data) && resend_codec_data)
+          rtph264pay->send_spspps = TRUE;
+      }
+      break;
+    default:
+      break;
+  }
+
+  return FALSE;
+}
+
+static GstStateChangeReturn
+gst_basertppayload_change_state (GstElement * element,
+    GstStateChange transition)
+{
+  GstStateChangeReturn ret;
+  GstRtpH264Pay *rtph264pay = GST_RTP_H264_PAY (element);
+
+  switch (transition) {
+    case GST_STATE_CHANGE_READY_TO_PAUSED:
+      rtph264pay->send_spspps = FALSE;
+      break;
+    default:
+      break;
+  }
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+
+  return ret;
+}
+
 static void
 gst_rtp_h264_pay_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
