@@ -49,6 +49,7 @@ struct _GstRtpMP4GDepay
 
   gint constantSize;
   gint constantDuration;
+  gint maxDisplacement;
 
   gint sizelength;
   gint indexlength;
@@ -75,6 +76,8 @@ struct _GstRtpMP4GDepayClass
   GstBaseRTPDepayloadClass parent_class;
 };
 
+GType gst_rtp_mp4g_depay_get_type (void);
+
 gboolean gst_rtp_mp4g_depay_plugin_init (GstPlugin * plugin);
 
 G_END_DECLS
