@@ -117,6 +117,7 @@ struct _GstRTSPStream {
 
   /* state */
   gint          pt;
+  guint         port;
   gboolean      container;
   /* original control url */
   gchar        *control_url;
@@ -130,6 +131,10 @@ struct _GstRTSPStream {
   guint         as_bandwidth;
   guint         rs_bandwidth;
   guint         rr_bandwidth;
+
+  /* destination */
+  gchar        *destination;
+  guint         ttl;
 };
 
 /**
